@@ -9,18 +9,14 @@
 #   template content.
 #
 # AUTHOR
-#   Andy Wardley   <abw@kfs.org>
+#   Andy Wardley   <abw@wardley.org>
 #
 # COPYRIGHT
-#   Copyright (C) 1996-2000 Andy Wardley.  All Rights Reserved.
-#   Copyright (C) 1998-2000 Canon Research Centre Europe Ltd.
+#   Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
 #
-# REVISION
-#   $Id: Document.pm,v 2.79 2006/05/30 17:01:28 abw Exp $
-#
 #============================================================================
 
 package Template::Document;
@@ -66,7 +62,7 @@ BEGIN {
 
 sub new {
     my ($class, $doc) = @_;
-    my ($block, $defblocks, $metadata) = @$doc{ qw( BLOCK DEFBLOCKS METADATA ) };
+    my ($block, $defblocks, $variables, $metadata) = @$doc{ qw( BLOCK DEFBLOCKS VARIABLES METADATA ) };
     $defblocks ||= { };
     $metadata  ||= { };
 
@@ -97,6 +93,7 @@ sub new {
         %$metadata,
         _BLOCK     => $block,
         _DEFBLOCKS => $defblocks,
+        _VARIABLES => $variables,
         _HOT       => 0,
     }, $class;
 }
@@ -128,6 +125,17 @@ sub blocks {
 }
 
 
+#-----------------------------------------------------------------------
+# variables()
+#
+# Returns a reference to a hash of variables used in the template.  
+# This requires the TRACE_VARS option to be enabled.
+#-----------------------------------------------------------------------
+
+sub variables {
+    return $_[0]->{ _VARIABLES };
+}
+
 #------------------------------------------------------------------------
 # process($context)
 #
@@ -160,7 +168,7 @@ sub process {
 
     die $context->catch($@)
         if $@;
-	
+        
     return $output;
 }
 
@@ -204,9 +212,9 @@ sub _dump {
     $output .= "BLOCK: $self->{ _BLOCK }\nDEFBLOCKS:\n";
 
     if ($dblks = $self->{ _DEFBLOCKS }) {
-	foreach my $b (keys %$dblks) {
-	    $output .= "    $b: $dblks->{ $b }\n";
-	}
+        foreach my $b (keys %$dblks) {
+            $output .= "    $b: $dblks->{ $b }\n";
+        }
     }
 
     return $output;
@@ -235,12 +243,12 @@ sub as_perl {
     my ($class, $content) = @_;
     my ($block, $defblocks, $metadata) = @$content{ qw( BLOCK DEFBLOCKS METADATA ) };
 
-    $block =~ s/\n/\n    /g;
+    $block =~ s/\n(?!#line)/\n    /g;
     $block =~ s/\s+$//;
 
     $defblocks = join('', map {
         my $code = $defblocks->{ $_ };
-        $code =~ s/\n/\n        /g;
+        $code =~ s/\n(?!#line)/\n        /g;
         $code =~ s/\s*$//;
         "        '$_' => $code,\n";
     } keys %$defblocks);
@@ -287,7 +295,7 @@ sub write_perl_file {
     my ($fh, $tmpfile);
     
     return $class->error("invalid filename: $file")
-	unless $file =~ /^(.+)$/s;
+        unless $file =~ /^(.+)$/s;
 
     eval {
         require File::Temp;
@@ -325,18 +333,6 @@ sub catch_warnings {
 
 __END__
 
-
-#------------------------------------------------------------------------
-# IMPORTANT NOTE
-#   This documentation is generated automatically from source
-#   templates.  Any changes you make here may be lost.
-# 
-#   The 'docsrc' documentation source bundle is available for download
-#   from http://www.template-toolkit.org/docs.html and contains all
-#   the source templates, XML files, scripts, etc., from which the
-#   documentation for the Template Toolkit is built.
-#------------------------------------------------------------------------
-
 =head1 NAME
 
 Template::Document - Compiled template document object
@@ -344,56 +340,59 @@ Template::Document - Compiled template document object
 =head1 SYNOPSIS
 
     use Template::Document;
-
+    
     $doc = Template::Document->new({
-	BLOCK => sub { # some perl code; return $some_text },
-	DEFBLOCKS => {
-	    header => sub { # more perl code; return $some_text },
-	    footer => sub { # blah blah blah; return $some_text },
-	},
-	METADATA => {
-	    author  => 'Andy Wardley',
-	    version => 3.14,
-	}
+        BLOCK => sub { # some perl code; return $some_text },
+        DEFBLOCKS => {
+            header => sub { # more perl code; return $some_text },
+            footer => sub { # blah blah blah; return $some_text },
+        },
+        METADATA => {
+            author  => 'Andy Wardley',
+            version => 3.14,
+        }
     }) || die $Template::Document::ERROR;
-
+    
     print $doc->process($context);
 
 =head1 DESCRIPTION
 
 This module defines an object class whose instances represent compiled
-template documents.  The Template::Parser module creates a
-Template::Document instance to encapsulate a template as it is compiled
+template documents.  The L<Template::Parser> module creates a
+C<Template::Document> instance to encapsulate a template as it is compiled
 into Perl code.
 
-The constructor method, new(), expects a reference to a hash array
-containing the BLOCK, DEFBLOCKS and METADATA items.  The BLOCK item
-should contain a reference to a Perl subroutine or a textual
-representation of Perl code, as generated by the Template::Parser
-module, which is then evaluated into a subroutine reference using
-eval().  The DEFLOCKS item should reference a hash array containing
-further named BLOCKs which may be defined in the template.  The keys
-represent BLOCK names and the values should be subroutine references
-or text strings of Perl code as per the main BLOCK item.  The METADATA
-item should reference a hash array of metadata items relevant to the
-document.
+The constructor method, L<new()>, expects a reference to a hash array
+containing the C<BLOCK>, C<DEFBLOCKS> and C<METADATA> items.  
 
-The process() method can then be called on the instantiated
-Template::Document object, passing a reference to a Template::Content
-object as the first parameter.  This will install any locally defined
-blocks (DEFBLOCKS) in the the contexts() BLOCKS cache (via a call to
-visit()) so that they may be subsequently resolved by the context.  The 
-main BLOCK subroutine is then executed, passing the context reference
-on as a parameter.  The text returned from the template subroutine is
-then returned by the process() method, after calling the context leave()
-method to permit cleanup and de-registration of named BLOCKS previously
-installed.
+The C<BLOCK> item should contain a reference to a Perl subroutine or a textual
+representation of Perl code, as generated by the L<Template::Parser> module.
+This is then evaluated into a subroutine reference using C<eval()>. 
 
-An AUTOLOAD method provides access to the METADATA items for the document.
-The Template::Service module installs a reference to the main 
-Template::Document object in the stash as the 'template' variable.
-This allows metadata items to be accessed from within templates, 
-including PRE_PROCESS templates.
+The C<DEFLOCKS> item should reference a hash array containing further named
+C<BLOCK>s which may be defined in the template. The keys represent C<BLOCK>
+names and the values should be subroutine references or text strings of Perl
+code as per the main C<BLOCK> item. 
+
+The C<METADATA> item should reference a hash array of metadata items relevant
+to the document.
+
+The L<process()> method can then be called on the instantiated
+C<Template::Document> object, passing a reference to a L<Template::Context>
+object as the first parameter. This will install any locally defined blocks
+(C<DEFBLOCKS>) in the C<BLOCKS> cache in the context (via a call to
+L<visit()|Template::Context#visit()>) so that they may be subsequently
+resolved by the context. The main C<BLOCK> subroutine is then executed,
+passing the context reference on as a parameter. The text returned from the
+template subroutine is then returned by the L<process()> method, after calling
+the context L<leave()|Template::Context#leave()> method to permit cleanup and
+de-registration of named C<BLOCKS> previously installed.
+
+An C<AUTOLOAD> method provides access to the C<METADATA> items for the
+document. The L<Template::Service> module installs a reference to the main
+C<Template::Document> object in the stash as the C<template> variable. This allows
+metadata items to be accessed from within templates, including C<PRE_PROCESS>
+templates.
 
 header:
 
@@ -403,7 +402,7 @@ header:
     </head>
     ...
 
-Template::Document objects are usually created by the Template::Parser
+C<Template::Document> objects are usually created by the L<Template::Parser>
 but can be manually instantiated or sub-classed to provide custom
 template components.
 
@@ -415,50 +414,51 @@ Constructor method which accept a reference to a hash array containing the
 structure as shown in this example:
 
     $doc = Template::Document->new({
-	BLOCK => sub { # some perl code; return $some_text },
-	DEFBLOCKS => {
-	    header => sub { # more perl code; return $some_text },
-	    footer => sub { # blah blah blah; return $some_text },
-	},
-	METADATA => {
-	    author  => 'Andy Wardley',
-	    version => 3.14,
-	}
+        BLOCK => sub { # some perl code; return $some_text },
+        DEFBLOCKS => {
+            header => sub { # more perl code; return $some_text },
+            footer => sub { # blah blah blah; return $some_text },
+        },
+        METADATA => {
+            author  => 'Andy Wardley',
+            version => 3.14,
+        }
     }) || die $Template::Document::ERROR;
 
-BLOCK and DEFBLOCKS items may be expressed as references to Perl subroutines
+C<BLOCK> and C<DEFBLOCKS> items may be expressed as references to Perl subroutines
 or as text strings containing Perl subroutine definitions, as is generated
-by the Template::Parser module.  These are evaluated into subroutine references
-using eval().
+by the L<Template::Parser> module.  These are evaluated into subroutine references
+using C<eval()>.
 
-Returns a new Template::Document object or undef on error.  The error() class
-method can be called, or the $ERROR package variable inspected to retrieve
-the relevant error message.
+Returns a new C<Template::Document> object or C<undef> on error. The
+L<error()|Template::Base#error()> class method can be called, or the C<$ERROR>
+package variable inspected to retrieve the relevant error message.
 
 =head2 process($context)
 
-Main processing routine for the compiled template document.  A reference to 
-a Template::Context object should be passed as the first parameter.  The 
-method installs any locally defined blocks via a call to the context 
-visit() method, processes it's own template, passing the context reference
-by parameter and then calls leave() in the context to allow cleanup.
+Main processing routine for the compiled template document. A reference to a
+L<Template::Context> object should be passed as the first parameter. The
+method installs any locally defined blocks via a call to the context
+L<visit()|Template::Context#visit()> method, processes its own template,
+(passing the context reference as a parameter) and then calls
+L<leave()|Template::Context#leave()> in the context to allow cleanup.
 
     print $doc->process($context);
 
 Returns a text string representing the generated output for the template.
-Errors are thrown via die().
+Errors are thrown via C<die()>.
 
 =head2 block()
 
-Returns a reference to the main BLOCK subroutine.
+Returns a reference to the main C<BLOCK> subroutine.
 
 =head2 blocks()
 
-Returns a reference to the hash array of named DEFBLOCKS subroutines.
+Returns a reference to the hash array of named C<DEFBLOCKS> subroutines.
 
 =head2 AUTOLOAD
 
-An autoload method returns METADATA items.
+An autoload method returns C<METADATA> items.
 
     print $doc->author();
 
@@ -467,40 +467,29 @@ An autoload method returns METADATA items.
 =head2 write_perl_file(\%config)
 
 This package subroutine is provided to effect persistence of compiled
-templates.  If the COMPILE_EXT option (to indicate a file extension
-for saving compiled templates) then the Template::Parser module calls
-this subroutine before calling the new() constructor.  At this stage,
+templates.  If the C<COMPILE_EXT> option (to indicate a file extension
+for saving compiled templates) then the L<Template::Parser> module calls
+this subroutine before calling the L<new()> constructor.  At this stage,
 the parser has a representation of the template as text strings
 containing Perl code.  We can write that to a file, enclosed in a
-small wrapper which will allow us to susequently require() the file
-and have Perl parse and compile it into a Template::Document.  Thus we
+small wrapper which will allow us to susequently C<require()> the file
+and have Perl parse and compile it into a C<Template::Document>.  Thus we
 have persistence of compiled templates.
 
 =head1 AUTHOR
 
-Andy Wardley E<lt>abw@wardley.orgE<gt>
-
-L<http://wardley.org/|http://wardley.org/>
-
-
-
-
-=head1 VERSION
-
-2.79, distributed as part of the
-Template Toolkit version 2.18, released on 09 February 2007.
+Andy Wardley E<lt>abw@wardley.orgE<gt> L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-  Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
-
+Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
-L<Template|Template>, L<Template::Parser|Template::Parser>
+L<Template>, L<Template::Parser>
 
 =cut
 
