@@ -119,6 +119,8 @@ sub new {
         FILE_INFO   => 1,
         GRAMMAR     => undef,
         _ERROR      => '',
+        IN_BLOCK    => [ ],
+        TRACE_VARS  => $config->{ TRACE_VARS },
         FACTORY     => $config->{ FACTORY } || 'Template::Directive',
     }, $class;
 
@@ -151,12 +153,13 @@ sub new {
         Template::Grammar->new();
     };
 
-    # build a FACTORY object to include any NAMESPACE definitions,
-    # but only if FACTORY isn't already an object
-    if ($config->{ NAMESPACE } && ! ref $self->{ FACTORY }) {
+    # instantiate a FACTORY object
+    unless (ref $self->{ FACTORY }) {
         my $fclass = $self->{ FACTORY };
-        $self->{ FACTORY } = $fclass->new( NAMESPACE => $config->{ NAMESPACE } )
-            || return $class->error($fclass->error());
+        $self->{ FACTORY } = $self->{ FACTORY }->new(
+             NAMESPACE => $config->{ NAMESPACE } 
+        )
+        || return $class->error($self->{ FACTORY }->error());
     }
     
     # load grammar rules, states and lex table
@@ -169,6 +172,42 @@ sub new {
     return $self;
 }
 
+#-----------------------------------------------------------------------
+# These methods are used to track nested IF and WHILE blocks.  Each 
+# generated if/while block is given a label indicating the directive 
+# type and nesting depth, e.g. FOR0, WHILE1, FOR2, WHILE3, etc.  The
+# NEXT and LAST directives use the innermost label, e.g. last WHILE3;
+#-----------------------------------------------------------------------
+
+sub enter_block {
+    my ($self, $name) = @_;
+    my $blocks = $self->{ IN_BLOCK };
+    push(@{ $self->{ IN_BLOCK } }, $name);
+}
+
+sub leave_block {
+    my $self = shift;
+    my $label = $self->block_label;
+    pop(@{ $self->{ IN_BLOCK } });
+    return $label;
+}
+
+sub in_block {
+    my ($self, $name) = @_;
+    my $blocks = $self->{ IN_BLOCK };
+    return @$blocks && $blocks->[-1] eq $name;
+}
+
+sub block_label {
+    my ($self, $prefix, $suffix) = @_;
+    my $blocks = $self->{ IN_BLOCK };
+    my $name   = @$blocks 
+        ? $blocks->[-1] . scalar @$blocks 
+        : undef;
+    return join('', grep { defined $_ } $prefix, $name, $suffix);
+}
+
+
 
 #------------------------------------------------------------------------
 # new_style(\%config)
@@ -238,8 +277,9 @@ sub parse {
 #    print "info: { ", join(', ', map { "$_ => $info->{ $_ }" } keys %$info), " }\n";
 
     # store for blocks defined in the template (see define_block())
-    my $defblock = $self->{ DEFBLOCK } = { };
-    my $metadata = $self->{ METADATA } = [ ];
+    my $defblock  = $self->{ DEFBLOCK  } = { };
+    my $metadata  = $self->{ METADATA  } = [ ];
+    my $variables = $self->{ VARIABLES } = { };
     $self->{ DEFBLOCKS } = [ ];
 
     $self->{ _ERROR } = '';
@@ -263,6 +303,7 @@ sub parse {
     return {
         BLOCK     => $block,
         DEFBLOCKS => $defblock,
+        VARIABLES => $variables,
         METADATA  => { @$metadata },
     };
 }
@@ -673,6 +714,7 @@ sub location {
     my $file = $info->{ path } || $info->{ name } 
         || '(unknown template)';
     $line =~ s/\-.*$//; # might be 'n-n'
+    $line ||= 1;
     return "#line $line \"$file\"\n";
 }
 
@@ -709,6 +751,12 @@ sub _parse {
     # retrieve internal rule and state tables
     my ($states, $rules) = @$self{ qw( STATES RULES ) };
 
+    # If we're tracing variable usage then we need to give the factory a 
+    # reference to our $self->{ VARIABLES } for it to fill in.  This is a
+    # bit of a hack to back-patch this functionality into TT2.
+    $self->{ FACTORY }->trace_vars($self->{ VARIABLES })
+        if $self->{ TRACE_VARS };
+
     # call the grammar set_factory method to install emitter factory
     $self->{ GRAMMAR }->install_factory($self->{ FACTORY });
 
