@@ -84,7 +84,7 @@ sub format {
             : ($params->{ gmt } || $self->{ gmt });
     my (@date, $datestr);
 
-    if ($time =~ /^\d+$/) {
+    if ($time =~ /^-?\d+$/) {
         # $time is now in seconds since epoch
         if ($gmt) {
             @date = (gmtime($time))[0..6];
